//+------------------------------------------------------------------+
//|                                    Spaced Lines ComoxBluejay.mq4 |
//+------------------------------------------------------------------+

#property indicator_chart_window

extern double  StartPrice     = 0;
extern int     NumLines       = 4;
extern int     Spacing        = 25;
extern string  AboveBelow     = "A";
extern color   LineColor      = Gray;
extern int     LineWidth      = 1;
extern int     LineStyle      = 0;
extern color   EntryLineColor = White;
extern int     EntryLineWidth = 1;
extern int     EntryLineStyle = 0;
extern string  TimeFrames     = "M1, M5, M15, M30, H1, H4, D1, W1, MN";

string   IndiName;

//+------------------------------------------------------------------+
int init()  {
//+------------------------------------------------------------------+
  int checksum = 0;
  string str = NumberToStr(StartPrice,"2.5") + NumberToStr(NumLines,"3") + NumberToStr(Spacing,"4.1") + AboveBelow + TimeToStr(TimeLocal(),TIME_SECONDS);
  for (int i=0; i<StringLen(str); i++)  
    checksum += i * StringGetChar(str,i);
  IndiName = "SpacedLines-" + checksum;
  IndicatorShortName(IndiName);

  del_obj();
  plot_obj();
  return(0);
}

//+------------------------------------------------------------------+
int deinit()  {
//+------------------------------------------------------------------+
  del_obj();
  return(0);
}

//+------------------------------------------------------------------+
int start()  {
//+------------------------------------------------------------------+
  return(0);
}

//+------------------------------------------------------------------+
int MathSign(double n)
//+------------------------------------------------------------------+
// Returns the sign of a number (i.e. -1, 0, +1)
// Usage:   int x=MathSign(-25)   returns x=-1
{
  if (n > 0) return(1);
  else if (n < 0) return (-1);
  else return(0);
}  

//+------------------------------------------------------------------+
double MathFix(double n, int d)
//+------------------------------------------------------------------+
// Returns N rounded to D decimals - works around a precision bug in MQL4
{
  return(MathRound(n*MathPow(10,d)+0.000000000001*MathSign(n))/MathPow(10,d));
}  


//+------------------------------------------------------------------+
void plot_obj()  {
//+------------------------------------------------------------------+

  TimeFrames = StringUpper(TimeFrames) + ",";
  int tf = 0;
  if (StringFind(TimeFrames,"M1,")  >= 0)    tf += 1;
  if (StringFind(TimeFrames,"M5,")  >= 0)    tf += 2;
  if (StringFind(TimeFrames,"M15,") >= 0)    tf += 4;
  if (StringFind(TimeFrames,"M30,") >= 0)    tf += 8;
  if (StringFind(TimeFrames,"H1,")  >= 0)    tf += 16;
  if (StringFind(TimeFrames,"H4,")  >= 0)    tf += 32;
  if (StringFind(TimeFrames,"D1,")  >= 0)    tf += 64;
  if (StringFind(TimeFrames,"W1,")  >= 0)    tf += 128;
  if (StringFind(TimeFrames,"MN,")  >= 0)    tf += 256;

  if (Digits >= 4)   {                            // non-JPY pair
     double mult = 0.0001;
     double prc  = MathFix(Close[0],2);
  }
  else  {                                         // JPY pair
     mult = 0.01;
     prc  = MathFix(Close[0],0);
  }   
  if (StartPrice > 0)
    prc = StartPrice;

  AboveBelow = StringUpper(AboveBelow);
  int X0 = 0, X1 = 0;
  if (AboveBelow == "A")    {  X0 = -1;           X1 = NumLines-2;  }     else
  if (AboveBelow == "B")    {  X0 = -NumLines+2;  X1 = 1;           }     else
  if (AboveBelow == "AB")   {  X0 = -NumLines+1;  X1 = NumLines-1;  }  
    
  for (int i=X0; i<=X1; i++)   {
    string objname = IndiName + "-" + i;
    ObjectCreate(objname,OBJ_HLINE,0,0,prc+Spacing*mult*i);
    ObjectSet(objname,OBJPROP_COLOR,LineColor);
    ObjectSet(objname,OBJPROP_WIDTH,LineWidth);
    ObjectSet(objname,OBJPROP_STYLE,LineStyle);
    if (i == X0+1 && AboveBelow == "A")   {
      ObjectSet(objname,OBJPROP_COLOR,EntryLineColor);
      ObjectSet(objname,OBJPROP_WIDTH,EntryLineWidth);
      ObjectSet(objname,OBJPROP_STYLE,EntryLineStyle);
    }  
    if (i == X1-1 && AboveBelow == "B")   {
      ObjectSet(objname,OBJPROP_COLOR,EntryLineColor);
      ObjectSet(objname,OBJPROP_WIDTH,EntryLineWidth);
      ObjectSet(objname,OBJPROP_STYLE,EntryLineStyle);
    }  
    ObjectSet(objname,OBJPROP_TIMEFRAMES,tf);
  }

}

//+------------------------------------------------------------------+
void del_obj()   {
//+------------------------------------------------------------------+
  int k=0;
  while (k<ObjectsTotal())   {
    string objname = ObjectName(k);
    if (StringSubstr(objname,0,StringLen(IndiName)) == IndiName)  
      ObjectDelete(objname);
    else
      k++;
  }    

}

//+------------------------------------------------------------------+
string StringUpper(string str)
//+------------------------------------------------------------------+
// Converts any lowercase characters in a string to uppercase
// Usage:    string x=StringUpper("The Quick Brown Fox")  returns x = "THE QUICK BROWN FOX"
{
  string outstr = "";
  string lower  = "abcdefghijklmnopqrstuvwxyz";
  string upper  = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  for(int i=0; i<StringLen(str); i++)  {
    int t1 = StringFind(lower,StringSubstr(str,i,1),0);
    if (t1 >=0)  
      outstr = outstr + StringSubstr(upper,t1,1);
    else
      outstr = outstr + StringSubstr(str,i,1);
  }
  return(outstr);
}  

//+------------------------------------------------------------------+
string NumberToStr(double n, string mask)
//+------------------------------------------------------------------+
// Formats a number using a mask, and return the resulting string
// Usage:    string result = NumberToStr(number,mask)
// 
// Mask parameters:
// n = number of digits to output, to the left of the decimal point
// n.d = output n digits to left of decimal point; d digits to the right
// -n.d = floating minus sign at left of output
// n.d- = mius sign at right of output
// +n.d = floating plus/minus sign at left of output
// ( or ) = enclose negative number in parentheses
// $ or  or  or  = include floating currency symbol at left of output
// , = use commas to separate thousands
// Z or z = left fill with zeros instead of spaces
// R or r = round result in rightmost displayed digit
// B or b = blank entire field if number is 0
// * = show asterisk in leftmost position if overflow occurs
// L or l = left align final string 

{
  mask = StringUpper(mask);
  int dotadj = 0;
  int dot    = StringFind(mask,".",0);
  if (dot < 0)  {
    dot    = StringLen(mask);
    dotadj = 1;
  }  

  int nleft  = 0;
  int nright = 0;
  for (int i=0; i<dot; i++)  {
    string tchar = StringSubstr(mask,i,1);
    if (tchar >= "0" && tchar <= "9")   nleft = 10 * nleft + StrToInteger(tchar);
  }
  if (dotadj == 0)   {
    for (i=dot+1; i<=StringLen(mask); i++)  {
      tchar = StringSubstr(mask,i,1);
      if (tchar >= "0" && tchar <= "9")  nright = 10 * nright + StrToInteger(tchar);
  } }
  nright = MathMin(nright,7);

  if (dotadj == 1)  {
    for (i=0; i<StringLen(mask); i++)  {
      tchar = StringSubstr(mask,i,1);
      if (tchar >= "0" && tchar <= "9")  {
        dot = i;
        break;
  } } }

  string csym = "";
  if (StringFind(mask,"$",0) >= 0)   csym = "$";
  if (StringFind(mask,"",0) >= 0)   csym = "";
  if (StringFind(mask,"",0) >= 0)   csym = "";
  if (StringFind(mask,"",0) >= 0)   csym = "";

  string leadsign  = "";
  string trailsign = "";
  if (StringFind(mask,"+",0) >= 0 && StringFind(mask,"+",0) < dot)  {
    leadsign = " ";
    if (n > 0)   leadsign  = "+";
    if (n < 0)   leadsign  = "-";
  }    
  if (StringFind(mask,"-",0) >= 0 && StringFind(mask,"-",0) < dot)
    if (n < 0)  leadsign  = "-"; else leadsign = " ";
  if (StringFind(mask,"-",0) >= 0 && StringFind(mask,"-",0) > dot)
    if (n < 0)  trailsign  = "-"; else trailsign = " ";
  if (StringFind(mask,"(",0) >= 0 || StringFind(mask,")",0) >= 0)  {
    leadsign  = " ";
    trailsign = " ";
    if (n < 0)  { 
      leadsign  = "("; 
      trailsign = ")";
  } }    

  if (StringFind(mask,",",0) >= 0) bool comma = true; else comma = false;
  if (StringFind(mask,"Z",0) >= 0) bool zeros = true; else zeros = false;
  if (StringFind(mask,"B",0) >= 0) bool blank = true; else blank = false;
  if (StringFind(mask,"R",0) >= 0) bool round = true; else round = false;
  if (StringFind(mask,"*",0) >= 0) bool overf = true; else overf = false;
  if (StringFind(mask,"L",0) >= 0) bool lftsh = true; else lftsh = false;
  if (StringFind(mask,";",0) >= 0) bool swtch = true; else swtch = false;

  if (round) n = MathFix(n,nright);
  string outstr = n;

  int dleft = 0;
  for (i=0; i<StringLen(outstr); i++)  {
    tchar = StringSubstr(outstr,i,1);
    if (tchar >= "0" && tchar <= "9")   dleft++;
    if (tchar == ".")   break;
  }
  
// Insert fill characters.......
  if (zeros) string fill = "0"; else fill = " ";
  if (n < 0)
    outstr = "-" + StringRepeat(fill,nleft-dleft) + StringSubstr(outstr,1,StringLen(outstr)-1);
  else  
    outstr = StringRepeat(fill,nleft-dleft) + StringSubstr(outstr,0,StringLen(outstr));

  outstr = StringSubstr(outstr,StringLen(outstr)-9-nleft,nleft+1+nright-dotadj);

// Insert the commas.......  
  if (comma)   {
    bool digflg = false;
    bool stpflg = false;
    string out1 = "";
    string out2 = "";
    for (i=0; i<StringLen(outstr); i++)  {
      tchar = StringSubstr(outstr,i,1);
      if (tchar == ".")   stpflg = true;
      if (!stpflg && (nleft-i == 3 || nleft-i == 6 || nleft-i == 9)) 
        if (digflg)   out1 = out1 + ","; else out1 = out1 + " "; 
      out1 = out1 + tchar;    
      if (tchar >= "0" && tchar <= "9")   digflg = true;
    }  
    outstr = out1;
  }  
// Add currency symbol and signs........  
  outstr = csym + leadsign + outstr + trailsign;

// 'Float' the currency symbol/sign.......
  out1 = "";
  out2 = "";
  bool fltflg = true;
  for (i=0; i<StringLen(outstr); i++)   {
    tchar = StringSubstr(outstr,i,1);
    if (tchar >= "0" && tchar <= "9")   fltflg = false;
    if ((tchar == " " && fltflg) || (blank && n == 0) )   out1 = out1 + " ";   else   out2 = out2 + tchar;
  }   
  outstr = out1 + out2;

// Overflow........  
  if (overf && dleft > nleft)  outstr = "*" + StringSubstr(outstr,1,StringLen(outstr)-1);

// Left shift.......
  if (lftsh)   {
    int len = StringLen(outstr);
    outstr = StringLeftTrim(outstr);
    outstr = outstr + StringRepeat(" ",len-StringLen(outstr)) ;
  }

// Switch period and comma.......
  if (swtch)   {
    out1 = "";
    for (i=0; i<StringLen(outstr); i++)   {
      tchar = StringSubstr(outstr,i,1);
      if (tchar == ".")   out1 = out1 + ",";     else
      if (tchar == ",")   out1 = out1 + ".";     else
      out1 = out1 + tchar;
    }    
    outstr = out1;
  }  

  return(outstr);
}

//+------------------------------------------------------------------+
string StringLeftTrim(string str)
//+------------------------------------------------------------------+
// Removes all leading spaces from a string
// Usage:    string x=StringLeftTrim("  XX YY  ")  returns x = "XX  YY  "
{
  bool   left = true;
  string outstr = "";
  for(int i=0; i<StringLen(str); i++)  {
    if (StringSubstr(str,i,1) != " " || !left) {
      outstr = outstr + StringSubstr(str,i,1);
      left = false;
  } }
  return(outstr);
}

//+------------------------------------------------------------------+
string StringRepeat(string str, int n)
//+------------------------------------------------------------------+
// Repeats the string STR N times
// Usage:    string x=StringRepeat("-",10)  returns x = "----------"
{
  string outstr = "";
  for(int i=0; i<n; i++)  {
    outstr = outstr + str;
  }
  return(outstr);
}

